@@ -194,6 +194,7 @@ static int      ps_io_fseek PT((FileData, int));
 static int      ps_io_ftell PT((FileData));
 
 static char    *readline PT((FileData, long, char **, long *, unsigned int *));
+static char    *readlineuntil PT((FileData, long, char **, long *, unsigned int *, char));
 static char    *gettextline PT((char *));
 static char    *ps_gettext PT((char *,char **));
 static int      blank PT((char *));
@@ -445,7 +446,7 @@ psscan(const char *filename, int scanstyle)
      * follows, this seems to be a real pjl file. */
     if(iscomment(line, "\033%-12345X@PJL")) {
         /* read until first DSC comment */
-        while(readline(fd, enddoseps, &line, &position, &line_len) && (line[0] != '%')) ;
+        readlineuntil(fd, enddoseps, &line, &position, &line_len, '%');
 	if(line[0] != '%') {
 	    fprintf(stderr, "psscan error: input files seems to be a PJL file.\n");
 	    ENDMESSAGE(psscan)
@@ -580,19 +581,16 @@ psscan(const char *filename, int scanstyle)
 	    } else {
 		int page_order;
 		
-		switch (sscanf(line+length("%%Pages:"), "%d %d",
+		switch (sscanf(line+length("%%Pages:"), "%u %d",
 			       &maxpages, &page_order)) {
 		    case 2:
 			if (page_order_set == NONE) {
 			    if (page_order == -1) {
 				doc->pageorder = DESCEND;
-				page_order_set = 1;
 			    } else if (page_order == 0) {
 				doc->pageorder = SPECIAL;
-				page_order_set = 1;
 			    } else if (page_order == 1) {
 				doc->pageorder = ASCEND;
-				page_order_set = 1;
 			    }
 			}
 		    case 1:
@@ -606,8 +604,7 @@ psscan(const char *filename, int scanstyle)
 	} else if (doc->nummedia == NONE &&
 		   iscomment(line+2, "DocumentMedia:")) {
 	    char w[21], h[21];
-	    doc->media = (Media)
-			 PS_malloc(sizeof (MediaStruct));
+	    doc->media = (Media) PS_calloc(1, sizeof (MediaStruct));
             CHECK_MALLOCED(doc->media);
 	    doc->media[0].name = ps_gettext(line+length("%%DocumentMedia:"),
 					    &next_char);
@@ -616,10 +613,12 @@ psscan(const char *filename, int scanstyle)
 		    doc->media[0].width = _spectre_strtod (w, NULL) + 0.5;
 		    doc->media[0].height = _spectre_strtod (h, NULL) + 0.5;
 		}
-		if (doc->media[0].width != 0 && doc->media[0].height != 0)
+		if (doc->media[0].width != 0 && doc->media[0].height != 0) {
 		    doc->nummedia = 1;
-		else
+		} else {
 		    PS_free(doc->media[0].name);
+		    doc->media[0].name = NULL;
+		}
 	    }
 	    preread=1;
 	    while (readline(fd, enddoseps, &line, &position, &line_len) &&
@@ -629,7 +628,8 @@ psscan(const char *filename, int scanstyle)
 			     PS_realloc(doc->media,
 				     (doc->nummedia+1)*
 				     sizeof (MediaStruct));
-                CHECK_MALLOCED(doc->media);
+		CHECK_MALLOCED(doc->media);
+		memset (doc->media + doc->nummedia, 0, sizeof (MediaStruct));
 		doc->media[doc->nummedia].name = ps_gettext(line+length("%%+"),
 							    &next_char);
 		if (doc->media[doc->nummedia].name != NULL) {
@@ -637,10 +637,12 @@ psscan(const char *filename, int scanstyle)
 		        doc->media[doc->nummedia].width = _spectre_strtod (w, NULL) + 0.5;
 			doc->media[doc->nummedia].height = _spectre_strtod (h, NULL) + 0.5;
 		    }
-		    if (doc->media[doc->nummedia].width != 0 &&
-			doc->media[doc->nummedia].height != 0) doc->nummedia++;
-		    else
+		    if (doc->media[doc->nummedia].width != 0 &&	doc->media[doc->nummedia].height != 0) {
+		        doc->nummedia++;
+		    } else {
 			PS_free(doc->media[doc->nummedia].name);
+			doc->media[doc->nummedia].name = NULL;
+		    }
 		}
 	    }
 	    section_len += line_len;
@@ -648,14 +650,11 @@ psscan(const char *filename, int scanstyle)
 	} else if (doc->nummedia == NONE &&
 		   iscomment(line+2, "DocumentPaperSizes:")) {
 
-	    doc->media = (Media)
-			 PS_malloc(sizeof (MediaStruct));
+	    doc->media = (Media) PS_calloc(1, sizeof (MediaStruct));
             CHECK_MALLOCED(doc->media);
 	    doc->media[0].name = ps_gettext(line+length("%%DocumentPaperSizes:"),
 					    &next_char);
 	    if (doc->media[0].name != NULL) {
-		doc->media[0].width = 0;
-		doc->media[0].height = 0;
 		for (i=0; papersizes[i].name; i++) {
 			dmp = (Media)&papersizes[i];
 		    /* Note: Paper size comment uses down cased paper size
@@ -672,10 +671,12 @@ psscan(const char *filename, int scanstyle)
 			break;
 		    }
 		}
-		if (doc->media[0].width != 0 && doc->media[0].height != 0)
+		if (doc->media[0].width != 0 && doc->media[0].height != 0) {
 		    doc->nummedia = 1;
-		else
+		} else {
 		    PS_free(doc->media[0].name);
+		    doc->media[0].name = NULL;
+		}
 	    }
 	    while ((cp = ps_gettext(next_char, &next_char))) {
 		doc->media = (Media)
@@ -683,9 +684,8 @@ psscan(const char *filename, int scanstyle)
 				     (doc->nummedia+1)*
 				     sizeof (MediaStruct));
                 CHECK_MALLOCED(doc->media);
+		memset (doc->media + doc->nummedia, 0, sizeof (MediaStruct));
 		doc->media[doc->nummedia].name = cp;
-		doc->media[doc->nummedia].width = 0;
-		doc->media[doc->nummedia].height = 0;
 		for (i=0; papersizes[i].name; i++) {
 			dmp = (Media)&papersizes[i];
 		    /* Note: Paper size comment uses down cased paper size
@@ -693,7 +693,7 @@ psscan(const char *filename, int scanstyle)
 		     * PaperSize comments.
 		     */
 		    if (_spectre_strcasecmp(doc->media[doc->nummedia].name,
-			       dmp->name) == 0) {
+					    dmp->name) == 0) {
 			PS_free(doc->media[doc->nummedia].name);
 			doc->media[doc->nummedia].name =
 				(char *)PS_malloc(strlen(dmp->name)+1);
@@ -704,10 +704,12 @@ psscan(const char *filename, int scanstyle)
 			break;
 		    }
 		}
-		if (doc->media[doc->nummedia].width != 0 &&
-		    doc->media[doc->nummedia].height != 0) doc->nummedia++;
-		else
+		if (doc->media[doc->nummedia].width != 0 && doc->media[doc->nummedia].height != 0) {
+		    doc->nummedia++;
+		} else {
 		    PS_free(doc->media[doc->nummedia].name);
+		    doc->media[doc->nummedia].name = NULL;
+		}
 	    }
 	    preread=1;
 	    while (readline(fd, enddoseps, &line, &position, &line_len) &&
@@ -720,9 +722,8 @@ psscan(const char *filename, int scanstyle)
 					 (doc->nummedia+1)*
 					 sizeof (MediaStruct));
                     CHECK_MALLOCED(doc->media);
+		    memset (doc->media + doc->nummedia, 0, sizeof (MediaStruct));
 		    doc->media[doc->nummedia].name = cp;
-		    doc->media[doc->nummedia].width = 0;
-		    doc->media[doc->nummedia].height = 0;
 		    for (i=0; papersizes[i].name; i++) {
 			    dmp = (Media)&papersizes[i];
 			/* Note: Paper size comment uses down cased paper size
@@ -730,16 +731,18 @@ psscan(const char *filename, int scanstyle)
 			 * PaperSize comments.
 			 */
 			if (_spectre_strcasecmp(doc->media[doc->nummedia].name,
-				   dmp->name) == 0) {
+						dmp->name) == 0) {
 			    doc->media[doc->nummedia].width = dmp->width;
 			    doc->media[doc->nummedia].height = dmp->height;
 			    break;
 			}
 		    }
-		    if (doc->media[doc->nummedia].width != 0 &&
-			doc->media[doc->nummedia].height != 0) doc->nummedia++;
-		    else
+		    if (doc->media[doc->nummedia].width != 0 && doc->media[doc->nummedia].height != 0) {
+		        doc->nummedia++;
+		    } else {
 			PS_free(doc->media[doc->nummedia].name);
+			doc->media[doc->nummedia].name = NULL;
+		    }
 		}
 	    }
 	    section_len += line_len;
@@ -858,6 +861,20 @@ psscan(const char *filename, int scanstyle)
 	          iscomment(line+2, "Page:") ||
 	          iscomment(line+2, "Trailer") ||
 	          iscomment(line+2, "EOF")))) {
+	    if (iscomment(line, "%!PS")) {
+	        /* Embedded document in Prolog, typically font resources.
+		 * Skip until end of resource or Prolog
+		 */
+		while (readline(fd, enddoseps, &line, &position, &line_len) &&
+		       !(DSCcomment(line) &&
+			 (iscomment(line+2, "EndProlog") ||
+			  iscomment(line+2, "BeginSetup") ||
+			  iscomment(line+2, "Page:") ||
+			  iscomment(line+2, "Trailer") ||
+			  iscomment(line+2, "EOF")))) {
+		    section_len += line_len;
+		}
+	    }
 	    if (!preread) section_len += line_len;
 	    preread = 0;
 	}
@@ -994,7 +1011,7 @@ newpage:
             CHECK_MALLOCED(doc->pages);
 	}
 	label = ps_gettext(line+length("%%Page:"), &next_char);
-	if (sscanf(next_char, "%d", &thispage) != 1) thispage = 0;
+	if (sscanf(next_char, "%u", &thispage) != 1) thispage = 0;
 	if (nextpage == 1) {
 	    ignore = thispage != 1;
 	}
@@ -1105,7 +1122,7 @@ continuepage:
 	    /* Do nothing */
 	} else if (iscomment(line+2, "Page:")) {
 	    PS_free(ps_gettext(line+length("%%Page:"), &next_char));
-	    if (sscanf(next_char, "%d", &thispage) != 1) thispage = 0;
+	    if (sscanf(next_char, "%u", &thispage) != 1) thispage = 0;
 	    if (!ignore && thispage == nextpage) {
 		if (doc->numpages > 0) {
 		    doc->pages[doc->numpages-1].end = position;
@@ -1751,23 +1768,6 @@ static char * readline (fd, enddoseps, lineP, positionP, line_lenP)
    }
    else
 #endif
-   if ((line[0] == '%') &&
-       positionP && *positionP > enddoseps &&
-       (!iscomment(line, "%!PS-AdobeFont") && (iscomment(line,"%!PS") || iscomment(line, "\004%!PS")))) {
-       nesting_level=1;
-       line = ps_io_fgetchars(fd,-1);
-       if (line) *line_lenP += FD_LINE_LEN;
-       while (line) {
-           if ((line[0] == '%') &&
-	       (iscomment(line,"%!PS") || iscomment(line, "\004%!PS")))
-	       nesting_level++;
-	   else if (IS_COMMENT("EOF"))
-	       nesting_level--;
-	   line = ps_io_fgetchars(fd,-1);
-	   if (line) *line_lenP += FD_LINE_LEN;
-	   if (nesting_level == 0) break;
-       }
-   } else 
    if  (!IS_COMMENT("Begin"))     {} /* Do nothing */
    else if IS_BEGIN("Document:")  {  /* Skip the EPS without handling its content */
        nesting_level=1;
@@ -1847,6 +1847,44 @@ static char * readline (fd, enddoseps, lineP, positionP, line_lenP)
    return(FD_BUF+FD_LINE_BEGIN);
 }
 
+/*----------------------------------------------------------*/
+/*
+   readlineuntil()
+   Read the next line in the postscript file until char is found
+   Similar to readline, but it doesn't skip lines.
+*/
+/*----------------------------------------------------------*/
+
+static char * readlineuntil (fd, enddoseps, lineP, positionP, line_lenP, charP)
+   FileData fd;
+   long enddoseps;
+   char **lineP;
+   long *positionP;
+   unsigned int *line_lenP;
+   char charP;
+{
+   char *line;
+
+   if (positionP) *positionP = FD_FILEPOS;
+
+   do {
+       line = ps_io_fgetchars(fd,-1);
+       if (!line) {
+           INFMESSAGE(could not get line)
+	   *line_lenP = 0;
+	   *lineP     = empty_string;
+	   ENDMESSAGE(readline)
+	   return(NULL);
+       }
+
+       if (positionP) *positionP = FD_FILEPOS;
+       *line_lenP = FD_LINE_LEN;
+       *lineP = FD_BUF+FD_LINE_BEGIN;
+   } while (line[0] != charP);
+
+   return(FD_BUF+FD_LINE_BEGIN);
+}
+
 /*###########################################################*/
 /*
  *	pscopyuntil -- copy lines of Postscript from a section of one file
@@ -2089,7 +2127,7 @@ pscopydoc(dest_file,src_filename,d,pagelist)
           fputs(comment, dest_file);
           pages_atend = True;
        } else {
-          switch (sscanf(comment+length("%%Pages:"), "%*d %d", &i)) {
+          switch (sscanf(comment+length("%%Pages:"), "%*d %u", &i)) {
              case 1:
                 fprintf(dest_file, "%%%%Pages: %d %d\n", pages, i);
                 break;
@@ -2124,7 +2162,7 @@ pscopydoc(dest_file,src_filename,d,pagelist)
          PS_free(comment);
          continue;
       }
-      switch (sscanf(comment+length("%%Pages:"), "%*d %d", &i)) {
+      switch (sscanf(comment+length("%%Pages:"), "%*d %u", &i)) {
          case 1:
             fprintf(dest_file, "%%%%Pages: %d %d\n", pages, i);
             break;
@@ -2233,7 +2271,7 @@ psgetpagebbox (const struct document *doc, int page, int *urx, int *ury, int *ll
    *urx = new_urx;
    *ury = new_ury;
 
-   return (new_llx != 0 && new_lly != 0 && new_urx != 0 && new_ury != 0);
+   return (new_llx != 0 || new_lly != 0 || new_urx != 0 || new_ury != 0);
 }
 
 /* From Evince */
