@@ -179,6 +179,7 @@ spectre_device_render (SpectreDevice        *device,
 	int        n_args = 12;
 	int        arg = 0;
 	int        success;
+	char      *fmt;
 	char      *text_alpha, *graph_alpha;
 	char      *size = NULL;
 	char      *resolution, *set;
@@ -215,7 +216,7 @@ spectre_device_render (SpectreDevice        *device,
 	args = calloc (sizeof (char *), n_args);
 	args[arg++] = "libspectre"; /* This value doesn't really matter */
 	args[arg++] = "-dMaxBitmap=10000000";
-	args[arg++] = "-dDELAYSAFER";
+	args[arg++] = "-dSAFER";
 	args[arg++] = "-dNOPAUSE";
 	args[arg++] = "-dNOPAGEPROMPT";
 	args[arg++] = "-sDEVICE=display";
@@ -239,8 +240,16 @@ spectre_device_render (SpectreDevice        *device,
 							   DISPLAY_LITTLEENDIAN |
 #endif
 							   DISPLAY_TOPFIRST);
-	args[arg++] = dsp_handle = _spectre_strdup_printf ("-sDisplayHandle=16#%llx",
-							   (uintptr_t)device);
+#ifdef WIN32
+#define FMT64 "I64"
+#else
+#define FMT64 "ll"
+#endif
+	fmt = _spectre_strdup_printf ("-sDisplayHandle=16#%s",
+				      sizeof (device) == 4 ? "%lx" : "%"FMT64"x");
+	args[arg++] = dsp_handle = _spectre_strdup_printf (fmt, device);
+	free (fmt);
+#undef FMT64
 	if (rc->use_platform_fonts == FALSE)
 		args[arg++] = "-dNOPLATFONTS";
 
